// Yet Another Units Library (YANL) - A C++ library for zero-overhead
// dimensional analysis and unit manipulation/conversion.
// 
// Copyright (C) 2003 Matthias Christian Schabel
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#ifndef MCS_ABC_UNITS_HPP
#define MCS_ABC_UNITS_HPP

#include <boost/mpl/list.hpp>

#include <mcs/units/dimensioned_quantity.hpp>

namespace mcs {

namespace units { 

struct a_tag { };
struct b_tag { };
struct c_tag { };

// basic units
typedef 
boost::mpl::list<dim<a_tag,Rational<1> > >     a_type;

typedef 
boost::mpl::list<dim<b_tag,Rational<1> > >     b_type;

typedef 
boost::mpl::list<dim<c_tag,Rational<1> > >     c_type;

// base class defining common units
template<class Model>
struct ABCUnitModel
{
	typedef DimensionedUnit<Model,a_type>	   a_unit;
	typedef DimensionedUnit<Model,b_type>	   b_unit;
	typedef DimensionedUnit<Model,c_type>	   c_unit;
};

struct ABCModel :
    public ABCUnitModel<ABCModel>
{
    typedef ABCModel    base_type; 
};

template<> 
struct UnitInfo<ABCModel,a_tag> 
{ 
    typedef ABCModel::base_type  base_type;

	static std::string symbol()				{ return "A"; }
	static std::string abbreviation()		{ return "a"; }
	
	template<typename Y> static Y toBase()	{ return Y(1); }
};
	
template<> 
struct UnitInfo<ABCModel,b_tag> 
{ 
    typedef ABCModel::base_type  base_type;

	static std::string symbol()				{ return "B"; }
	static std::string abbreviation()		{ return "b"; }
	
	template<typename Y> static Y toBase()	{ return Y(1); }
};
	
template<> 
struct UnitInfo<ABCModel,c_tag> 
{ 
    typedef ABCModel::base_type  base_type;

	static std::string symbol()				{ return "C"; }
	static std::string abbreviation()		{ return "c"; }
	
	template<typename Y> static Y toBase()	{ return Y(1); }
};
	
template<class Y,
         class Model>
struct ABCQuantityModel
{
    typedef DimensionedQuantity<Y,typename Model::a_unit>       A;
    typedef DimensionedQuantity<Y,typename Model::b_unit>       B;
    typedef DimensionedQuantity<Y,typename Model::c_unit>       C;
};

template<class Y>
struct ABC :
    public ABCQuantityModel<Y,ABCModel>
{ }; 

} // namespace units

} // namespace mcs

#endif // MCS_ABC_UNITS_HPP
