// Yet Another Units Library (YANL) - A C++ library for zero-overhead
// dimensional analysis and unit manipulation/conversion.
// 
// Copyright (C) 2003 Matthias Christian Schabel
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#ifndef MCS_DE_UNITS_HPP
#define MCS_DE_UNITS_HPP

#include <boost/mpl/list.hpp>

#include <mcs/units/dimensioned_quantity.hpp>

namespace mcs {

namespace units { 

struct d_tag { };
struct e_tag { };

// basic units
typedef 
boost::mpl::list<dim<d_tag,Rational<1> > >     d_type;

typedef 
boost::mpl::list<dim<e_tag,Rational<1> > >     e_type;

// base class defining common units
template<class Model>
struct DEUnitModel
{
	typedef DimensionedUnit<Model,d_type>	   d_unit;
	typedef DimensionedUnit<Model,e_type>	   e_unit;
};

struct DEModel :
    public DEUnitModel<DEModel>
{ 
    typedef DEModel base_type;
};

template<> 
struct UnitInfo<DEModel,d_tag> 
{ 
    typedef DEModel::base_type  base_type;

	static std::string symbol()				{ return "D"; }
	static std::string abbreviation()		{ return "d"; }
	
	template<typename Y> static Y toBase()	{ return Y(1); }
};
	
template<> 
struct UnitInfo<DEModel,e_tag> 
{ 
    typedef DEModel::base_type  base_type;

	static std::string symbol()				{ return "E"; }
	static std::string abbreviation()		{ return "e"; }
	
	template<typename Y> static Y toBase()	{ return Y(1); }
};
	
template<class Y,
         class Model>
struct DEQuantityModel
{
    typedef DimensionedQuantity<Y,typename Model::d_unit>       D;
    typedef DimensionedQuantity<Y,typename Model::e_unit>       E;
};

template<class Y>
struct DE :
    public DEQuantityModel<Y,DEModel>
{ };

} // namespace units

} // namespace mcs

#endif // MCS_DE_UNITS_HPP
