// Yet Another Units Library (YANL) - A C++ library for zero-overhead
// dimensional analysis and unit manipulation/conversion.
// 
// Copyright (C) 2003 Matthias Christian Schabel
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#ifndef MCS_FSF_UNITS_HPP
#define MCS_FSF_UNITS_HPP

#include <string>

#include <mcs/units/dimensioned_quantity.hpp>
#include <mcs/units/physical_units.hpp>

namespace mcs {

namespace units { 

class SIModel;

// furlong/stone/fortnight units
//
//    purely for pedagogical value, demonstrating how to set up a new system of units
struct FSFModel 
{ 
    typedef DimensionedUnit<FSFModel,length_type>                     length_unit;
    typedef DimensionedUnit<FSFModel,mass_type>                       mass_unit;
    typedef DimensionedUnit<FSFModel,time_type>                       time_unit;
};

template<> 
struct UnitInfo<FSFModel,length_tag> 
{ 
    typedef SIModel base_type;
    
	static std::string symbol()				{ return "furlong"; }
	static std::string abbreviation()		{ return "fl"; }
	
	template<typename Y> static Y toBase()	{ return Y(603.504); }
};
	
template<> 
struct UnitInfo<FSFModel,mass_tag> 
{ 
    typedef SIModel base_type;
    
	static std::string symbol()				{ return "stone"; }
	static std::string abbreviation()		{ return "st"; }
	
	template<typename Y> static Y toBase()	{ return Y(6.363636); }
};
	
template<> 
struct UnitInfo<FSFModel,time_tag> 
{ 
    typedef SIModel base_type;
    
	static std::string symbol()				{ return "fortnight"; }
	static std::string abbreviation()		{ return "fn"; }
	
	template<typename Y> static Y toBase()	{ return Y(1209600); }
};
	
namespace fsf {

static const FSFModel::length_unit          _furlong,
                                            _furlongs;
static const FSFModel::mass_unit            _stone;
static const FSFModel::time_unit            _fortnight,
                                            _fortnights;
                                                    
} // namespace fsf

template<class Y>
struct FSF
{
    typedef DimensionedQuantity<Y,typename FSFModel::length_unit>               Length;
    typedef DimensionedQuantity<Y,typename FSFModel::mass_unit>                 Mass;
    typedef DimensionedQuantity<Y,typename FSFModel::time_unit>                 Time;
};

} // namespace units

} // namespace mcs

#endif // MCS_FSF_UNITS_HPP
